package com.tzld.piaoquan.recommend.server.service.recall.strategy;

import com.google.common.collect.Lists;
import com.tzld.piaoquan.recommend.server.service.filter.FilterParam;
import com.tzld.piaoquan.recommend.server.service.filter.FilterResult;
import com.tzld.piaoquan.recommend.server.service.filter.FlowPoolWithLevelFilterService;
import com.tzld.piaoquan.recommend.server.service.filter.FlowPoolWithLevelScoreFilterService;
import com.tzld.piaoquan.recommend.server.service.recall.RecallParam;
import com.tzld.piaoquan.recommend.server.service.recall.RecallResult;
import com.tzld.piaoquan.recommend.server.service.recall.RecallStrategy;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ZSetOperations;

import java.util.*;

/**
 * @author dyp
 */
public abstract class AbstractFlowPoolWithLevelScoreRecallStrategy implements RecallStrategy {
    @Autowired
    protected RedisTemplate<String, String> redisTemplate;

    @Autowired
    protected FlowPoolWithLevelScoreFilterService flowPoolFilterService;

    @Override
    public RecallResult recall(RecallParam param) {
        Pair<String, String> flowPoolKeyAndLevel = flowPoolKeyAndLevel(param);
        String flowPoolKey = flowPoolKeyAndLevel.getLeft();
        String level = flowPoolKeyAndLevel.getRight();

        int idx = 0;
        int getSize = param.getSize() * 5;
        List<RecallResult.RecallData> results = new ArrayList<>();

        Set<ZSetOperations.TypedTuple<String>> data = redisTemplate.opsForZSet().reverseRangeWithScores(flowPoolKey, idx,
                idx + getSize - 1);
        if (CollectionUtils.isEmpty(data)) {
            return null;
        }
        idx += getSize;

        Map<Long, Double> videoScoreMap = new HashMap<>();
        Map<Long, String> videoFlowPoolMap = new HashMap<>();
        for (ZSetOperations.TypedTuple<String> t : data) {
            String[] values = t.getValue().split("-");
            videoFlowPoolMap.put(NumberUtils.toLong(values[0], 0), values[1]);
            videoScoreMap.put(NumberUtils.toLong(values[0], 0), t.getScore());
        }

        FilterParam filterParam = new FilterParam();
        filterParam.setVideoIds(Lists.newArrayList(videoFlowPoolMap.keySet()));
        FilterResult filterResult = flowPoolFilterService.filter(filterParam);

        if (filterResult != null && CollectionUtils.isNotEmpty(filterResult.getVideoIds())) {
            filterResult.getVideoIds().stream().forEach(vid -> {
                RecallResult.RecallData recallData = new RecallResult.RecallData();
                recallData.setVideoId(vid);
                recallData.setAbCode(param.getAbCode());
                recallData.setRovScore(videoScoreMap.get(vid));
                recallData.setPushFrom(pushFrom());
                recallData.setFlowPool(videoFlowPoolMap.get(vid));
                recallData.setFlowPoolAbtestGroup(param.getFlowPoolAbtestGroup());
                recallData.setLevel(level);
                results.add(recallData);
            });
        }

        RecallResult result = new RecallResult();
        result.setRecallData(results.subList(0, results.size() < param.getSize() ? results.size() : param.getSize()));
        return result;
    }

    abstract Pair<String, String> flowPoolKeyAndLevel(RecallParam param);

    abstract String pushFrom();

}
