package com.tzld.piaoquan.recommend.server.service.recall.strategy;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.tzld.piaoquan.recommend.server.service.filter.FilterParam;
import com.tzld.piaoquan.recommend.server.service.filter.FilterResult;
import com.tzld.piaoquan.recommend.server.service.filter.VideoFilterService;
import com.tzld.piaoquan.recommend.server.service.recall.RecallParam;
import com.tzld.piaoquan.recommend.server.service.recall.RecallResult;
import com.tzld.piaoquan.recommend.server.service.recall.RecallStrategy;
import lombok.Data;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
public abstract class AbstractVideoRecallStrategy implements RecallStrategy {
    @Autowired
    protected RedisTemplate<String, String> redisTemplate;

    @Autowired
    protected VideoFilterService simHotItemFilterService;

    @Override
    public RecallResult recall(RecallParam param) {


        String recall_key = recallKey(param);
        // [[16687915, 0.5], [16528093, 0.5]]
        String value = redisTemplate.opsForValue().get(recall_key);
        if (StringUtils.isBlank(value)) {
            return null;
        }
        List<Item> items = JSONObject.parseArray(value, Item.class);
        List<RecallResult.RecallData> results = new ArrayList<>();

        Map<Long, Double> videoScoreMap = new HashMap<>();
        for (int i = 0; i < items.size() && i < 20; i++) {
            Item item = items.get(i);
            videoScoreMap.put(NumberUtils.toLong(item.getVideoId(), 0),
                    NumberUtils.toDouble(item.getScore(), 0));
        }

        FilterParam filterParam = new FilterParam();
        filterParam.setVideoIds(Lists.newArrayList(videoScoreMap.keySet()));
        FilterResult filterResult = simHotItemFilterService.filter(filterParam);

        if (filterResult != null && CollectionUtils.isNotEmpty(filterResult.getVideoIds())) {
            filterResult.getVideoIds().stream().forEach(vid -> {
                RecallResult.RecallData recallData = new RecallResult.RecallData();
                recallData.setVideoId(vid);
                recallData.setAbCode(param.getAbCode());
                recallData.setRovScore(videoScoreMap.get(vid));
                recallData.setPushFrom(pushFrom());
                recallData.setFlowPool("");
                results.add(recallData);
            });
        }

        RecallResult result = new RecallResult();
        result.setRecallData(results);
        return result;
    }

    @Data
    class Item {
        private String videoId;
        private String score;
    }

    abstract String recallKey(RecallParam param);
    abstract String pushFrom();
}
